@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # <sure - utility belt for automated testing in python>
 # Copyright (C) <2010-2013>  Gabriel Falcão <gabriel@nacaolivre.org>
@@ -25,10 +24,10 @@ import difflib
 import inspect
 import traceback
 
-from functools import wraps
+from functools import wraps, partial
 from datetime import datetime
 
-from six import string_types, text_type, PY3, get_function_code
+from six import string_types, text_type, PY2, get_function_code
 from six.moves import reduce
 
 from sure.old import AssertionHelper
@@ -40,27 +39,16 @@ from sure.core import DeepExplanation
 from sure.core import _get_file_name
 from sure.core import _get_line_number
 from sure.core import safe_repr
+from sure.core import anything
 
 from sure.magic import is_cpython, patchable_builtin
 from sure.registry import context as _registry
 
 
-if PY3:
+if not PY2:
     basestring = str
 
-version = '1.3.0'
-
-
-def print_py26_deprecation_warn():
-    # warn the user if sure is used with python 2.6.
-    # sure will remove python 2.6 support in the next version.
-    # make sure to remove this code as soon as the python 2.6 support is removed
-    if sys.version_info[0] == 2 and sys.version_info[1] == 6:
-        import warnings
-        warnings.simplefilter("always", PendingDeprecationWarning)
-        warnings.warn("The next version of sure will NO LONGER support python 2.6", PendingDeprecationWarning)
-
-print_py26_deprecation_warn()
+version = '1.4.1'
 
 
 not_here_error = \
@@ -197,11 +185,11 @@ def within(**units):
             try:
                 func(start, *args, **kw)
             except TypeError as e:
-                if PY3:
-                    # PY3 has different error message
-                    fmt = '{0}() takes 0 positional arguments but 1 was given'
-                else:
+                if PY2:
+                    # PY2 has different error message
                     fmt = '{0}() takes no arguments'
+                else:
+                    fmt = '{0}() takes 0 positional arguments but 1 was given'
                 err = text_type(e)
                 if fmt.format(func.__name__) in err:
                     func(*args, **kw)
@@ -399,7 +387,7 @@ def assertionmethod(func):
             ", ".join(map(safe_repr, args)),
             ", ".join(["{0}={1}".format(k, safe_repr(kw[k])) for k in kw]),
         )
-        if not PY3:
+        if PY2:
             msg = text_type(msg)
 
         assert value, msg
@@ -650,6 +638,14 @@ class AssertionBuilder(object):
 
     @assertionmethod
     def equal(self, what, epsilon=None):
+        """compares given object ``X``  with an expected ``Y`` object.
+
+        It primarily assures that the compared objects are absolute equal ``==``.
+
+        :param what: the expected value
+        :param epsilon: a delta to leverage upper-bound floating point permissiveness
+        """
+
         try:
             comparison = DeepComparison(self.obj, what, epsilon).compare()
             error = False
@@ -925,30 +921,87 @@ those = AssertionBuilder('those')
 expect = AssertionBuilder('expect')
 
 
+def assertion(func):
+    """Extend sure with a custom assertion method."""
+    func = assertionmethod(func)
+    setattr(AssertionBuilder, func.__name__, func)
+    return func
+
+
+def chain(func):
+    """Extend sure with a custom chaining method."""
+    setattr(AssertionBuilder, func.__name__, func)
+    return func
+
+
+def chainproperty(func):
+    """Extend sure with a custom chain property."""
+    func = assertionproperty(func)
+    setattr(AssertionBuilder, func.fget.__name__, func)
+    return func
+
+
 allows_new_syntax = not os.getenv('SURE_DISABLE_NEW_SYNTAX')
 
 
 if is_cpython and allows_new_syntax:
-
     def make_safe_property(method, name, should_be_property=True):
         if not should_be_property:
             return method(None)
 
-        def deleter(self, *args, **kw):
-            pass
+        def deleter(method, self, *args, **kw):
+            if isinstance(self, type):
+                # if the attribute has to be deleted from a class object
+                # we cannot use ``del self.__dict__[name]`` directly because we cannot
+                # modify a mappingproxy object. Thus, we have to delete it in our
+                # proxy __dict__.
+                del overwritten_object_handlers[(id(self), method.__name__)]
+            else:
+                # if the attribute has to be deleted from an instance object
+                # we are able to directly delete it from the object's __dict__.
+                del self.__dict__[name]
 
-        def setter(self, other):
-            pass
+        def setter(method, self, other):
+            if isinstance(self, type):
+                # if the attribute has to be set to a class object
+                # we cannot use ``self.__dict__[name] = other`` directly because we cannot
+                # modify a mappingproxy object. Thus, we have to set it in our
+                # proxy __dict__.
+                overwritten_object_handlers[(id(self), method.__name__)] = other
+            else:
+                # if the attribute has to be set to an instance object
+                # we are able to directly set it in the object's __dict__.
+                self.__dict__[name] = other
 
         return builtins.property(
             fget=method,
-            fset=setter,
-            fdel=deleter,
+            fset=partial(setter, method),
+            fdel=partial(deleter, method),
         )
 
-    def positive_assertion(name, prop=True):
+
+    def build_assertion_property(name, is_negative, prop=True):
+        """Build assertion property
+
+        This is the assertion property which is usually patched
+        to the built-in ``object`` and ``NoneType``.
+        """
         def method(self):
-            builder = AssertionBuilder(name, negative=False)
+            # check if the given object already has an attribute with the
+            # given name. If yes return it instead of patching it.
+            try:
+                if name in self.__dict__:
+                    return self.__dict__[name]
+            except AttributeError:
+                # we do not have an object with __dict__, thus
+                # it's safe to just continue and patch the `name`.
+                pass
+
+            overwritten_object_handler = overwritten_object_handlers.get((id(self), name), None)
+            if overwritten_object_handler:
+                return overwritten_object_handler
+
+            builder = AssertionBuilder(name, negative=is_negative)
             instance = builder(self)
             callable_args = getattr(self, '_callable_args', ())
             if callable_args:
@@ -961,46 +1014,46 @@ if is_cpython and allows_new_syntax:
         method.__name__ = str(name)
         return make_safe_property(method, name, prop)
 
-    def negative_assertion(name, prop=True):
-        def method(self):
-            builder = AssertionBuilder(name, negative=True)
-            instance = builder(self)
-            callable_args = getattr(self, '_callable_args', ())
-            if callable_args:
-                instance._callable_args = callable_args
-            callable_kw = getattr(self, '_callable_kw', {})
-            if callable_kw:
-                instance._callable_kw = callable_kw
-            return instance
-
-        method.__name__ = str(name)
-        return make_safe_property(method, name, prop)
 
     object_handler = patchable_builtin(object)
+    # We have to keep track of all objects which
+    # should overwrite a ``POSITIVES`` or ``NEGATIVES``
+    # property. If we wouldn't do that in the
+    # make_safe_property.setter method we would loose
+    # the newly assigned object reference.
+    overwritten_object_handlers = {}
 
     # None does not have a tp_dict associated to its PyObject, so this
     # is the only way we could make it work like we expected.
     none = patchable_builtin(None.__class__)
 
     for name in POSITIVES:
-        object_handler[name] = positive_assertion(name)
-        none[name] = positive_assertion(name, False)
+        object_handler[name] = build_assertion_property(name, is_negative=False)
+        none[name] = build_assertion_property(name, is_negative=False, prop=False)
 
     for name in NEGATIVES:
-        object_handler[name] = negative_assertion(name)
-        none[name] = negative_assertion(name, False)
+        object_handler[name] = build_assertion_property(name, is_negative=True)
+        none[name] = build_assertion_property(name, is_negative=True, prop=False)
 
 
 old_dir = dir
 
 
-if allows_new_syntax:
+def enable():
     @wraps(builtins.dir)
-    def _new_dir(obj=None):
-        if obj is None:
+    def _new_dir(*obj):
+        if not obj:
             frame = inspect.currentframe()
             return sorted(frame.f_back.f_locals.keys())
-        else:
-            return sorted(set(old_dir(obj)).difference(POSITIVES + NEGATIVES))
+
+        if len(obj) > 1:
+            raise TypeError('dir expected at most 1 arguments, got {0}'.format(len(obj)))
+
+        patched = [x for x in old_dir(obj[0]) if isinstance(getattr(obj[0], x, None), AssertionBuilder)]
+        return sorted(set(old_dir(obj[0])).difference(patched))
 
     builtins.dir = _new_dir
+
+
+if allows_new_syntax:
+    enable()
